# bson-schema-to-typescript

Compile MongoDB JSON Schema to TypeScript typings. This package connects to your
MongoDB server, retrieves any stored JSON Schema validators, and compiles them
into TypeScript typings.

For more information about MongoDB schema validation, see:

[MongoDB Schema
Validation](https://docs.mongodb.com/manual/core/schema-validation/#json-schema)
and [MongoDB \$jsonSchema
operator](https://docs.mongodb.com/manual/reference/operator/query/jsonSchema/)

## Features

Written in TypeScript and includes type definitions.

- Can be used as CLI or programmatically
- Works with vanilla JavaScript and
  [TypeScript](https://www.typescriptlang.org/)
- Adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html)
- CLI output is formatted with [Prettier](https://prettier.io/), adhering to
  project [format options](https://prettier.io/docs/en/configuration.html)

## Installation

```sh
# Yarn:
yarn add -D bson-schema-to-typescript

# NPM:
npm install bson-schema-to-typescript --save-dev
```

## Configuration

### Parser configuration (optional)

Create a `bson2ts.json` configuration file in the root of your project. The
following values are the current defaults.

The `env` section enables mapping of your MongoDB environment variables, see the
next section below.

```json
{
  "bannerComment": [
    "/* eslint-disable */",
    "/* tslint:disable */",
    "/**",
    "* This file was automatically generated by bson-schema-to-typescript.",
    "* Do not modify it by hand. Instead, modify the MongoDB $jsonSchema validator,",
    "* and run bson2ts to regenerate this file.",
    "*/"
  ],
  "enableConstEnums": true,
  "ignoreMinAndMaxItems": false,
  "strictIndexSignatures": false,
  "unknownAny": true,
  "path": "src/__generated__",
  "env": {
    "MONGODB_URI": "MONGODB_URI",
    "MONGODB_DATABASE": "MONGODB_DATABASE"
  }
}
```

### MongoDB configuration (required for CLI usage)

The CLI connects to your MongoDB instance using two environment variables, the
MongoDB connection string (ex. `mongodb://localhost:27017`), and the MongoDB
database name (ex. `my-app`).

The CLI will by default use `MONGODB_URI` and `MONGODB_DATABASE` respectively,
if no configuration is provided.

If you already have these environment variables configured under different
names, you can make the CLI aware of that by providing the `env` section of the
`bson2ts.json` configuration file.

For example

```json
{
  "env": {
    "MONGODB_URI": "MY_MONGODB_URL",
    "MONGODB_DATABASE": "MY_MONGODB_DB"
  }
}
```

## CLI usage

Generate typings for each MongoDB collection.

Using `npx`:

```sh
npx bson-schema-to-typescript
```

Or add a script to `package.json`:

```json
{
  "scripts": {
    "generate-types": "bson2ts"
  }
}

Then run `yarn generate-types` or `npm run generate-types`
```

## Usage of generated typings

Once you have generated the types, you can implement them like this:

```ts
import { Db } from "mongodb";
import { UserDoc } from "./__generated__/UserDoc";
import { PostDoc } from "./__generated__/PostDoc";

export async function collections(db: Db) {
  return {
    users: db.collection<UserDoc>("users"),
    posts: db.collection<PostDoc>("posts"),
  };
}
```

## Programmatic usage

```ts
import {
  compileBSON,
  getCollectionSchema,
  getAllServerSchemas,
} from "bson-schema-to-typescript";
```

## Maintainers

- [@lirbank](https://github.com/lirbank)
